#include <SoftwareSerial.h>
#include <AccelStepper.h>
#include <math.h>
SoftwareSerial BTSerial(10, 11);   // RX | TX
// defines pins numbers

//Global Variable Definitions
const int StepMSlider = 3; //Step pin for motor slider
const int DirMSlider = 2;  //Direction pin for motorslider
const int MS1Slider = 5;
const int MS2Slider = 4;


const int StepPan = 7; //Step pin for motor slider
const int DirPan = 6;  //Direction pin for motorslider
const int MS1Pan = 9;
const int MS2Pan = 8; //Microstepping pins

const int originendstop = 13;
const int farendstop = 12;

//const int ledpin = 13;

///https://lastminuteengineers.com/a4988-stepper-motor-driver-arduino-tutorial/
//https://www.brainy-bits.com/microstepping-stepper-motor/
//https://www.arduino.cc/reference/en/language/structure/arithmetic-operators/modulo/

const byte NumChars = 12;
const byte NumParTempMove = 2;


//Motor Interfaces
const int motorInterfaceType = 1;

//Create Motor Instances
AccelStepper MotorSlider(motorInterfaceType, StepMSlider, DirMSlider);
AccelStepper MotorPan(motorInterfaceType, StepPan, DirPan);

//AccelStepper SliderStepper(motorInterfaceType,StepMSlider,DirMSlider);
//AccelStepper PanStepper(motorInterfaceType,StepPan,DirPan);

int SliderSpeed = 4000;//Varies from fastest at 500 to 2500.
int ReturnOriginSpeed = 4000;
long CalibrationSliderPos;
int SliderMoveDist;
int PanSpeed = 100;
int PanAngle = 0;
int CalibrationPanPos = 1596 * 2; //Microstepping Mode is now 1/16th
int MoveSpeed = 0;
int FeasiblePan = CalibrationPanPos / 3;
int CalibrationRestTime = 500;
//int SlidertoPanRatio;
int ManualMovePosDistance = 0;
int ManualMoveOldPosDistance = 0;
int ManualMovePosAngle = 0;
char TempManMove[NumParTempMove] = {0, 0};
char TempManPan[NumParTempMove];
char BTFeed[NumChars];
char BTData[NumChars];
char BTOldData[NumChars];
float LengthToSpeedMultiplier = 0;
bool TimeLapseMode = false;
boolean newData = false;
char TempSpeed[2];
int SpeedScalar = 70;//CHANGE TO HIGHER IF YOU WANT THE MOUNT TO RUN SLOWER MIN SPEED OR LOWER IF YOU WANT IT TO GO FASTER MAX


void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);//Initialise main Serial line

  BTSerial.begin(9600);//Initialise Bluetooth Serial
  BTSerial.print("Bluetooth Connected");
  delay(1000);
  pinMode(StepMSlider, OUTPUT); //Motor Slider Step Pin
  pinMode(DirMSlider, OUTPUT); //Motor Slider Direction Pin
  pinMode(StepPan, OUTPUT);
  pinMode(DirPan, OUTPUT);
  pinMode(originendstop, INPUT);
  pinMode(farendstop, INPUT);
  pinMode(MS1Slider, OUTPUT);
  pinMode(MS2Slider, OUTPUT);
  pinMode(MS1Pan, OUTPUT);
  pinMode(MS2Pan, OUTPUT);
  MotorSlider.setMaxSpeed(64000);
  MotorSlider.setAcceleration(5000);
  MotorPan.setMaxSpeed(4000);
  MotorPan.setAcceleration(500);
  digitalWrite(MS1Slider, HIGH);
  digitalWrite(MS2Slider, HIGH);
  digitalWrite(MS1Pan, HIGH);
  digitalWrite(MS2Pan, HIGH);

}

void recvWithEndMarker() {//This Recieved Data with endmarkers form the BTSerial
  static byte ndx = 0;
  char endMarker = 'X';
  char rc;

  while (BTSerial.available() > 0 && newData == false) {
    rc = BTSerial.read();

    if (rc != endMarker) {
      BTFeed[ndx] = rc;
      ndx++;
      if (ndx >= NumChars) {
        ndx = NumChars - 1;
      }
    }
    else {
      BTFeed[ndx] = '\0'; // terminate the string
      ndx = 0;
      newData = true;
    }
  }
}

char CheckNewData() {
  if (newData == true) {
    //Serial.println(BTFeed[0]);
    strcpy(BTData, BTFeed);
    //Serial.print("Old Dat is");
    //Serial.println(BTOldData);
    //Serial.print("New Data is");
    //Serial.println(BTData);

    if (BTData[0] != BTOldData[0]) {
      ///Serial.println(BTData[0]);
      //Serial.println(BTOldData[0]);
      //strcpy(BTData, BTFeed);
      //Serial.println(BTFeed);

      switch (BTData[0])
      {
        case 'C'://Calubrate
          strcpy(BTOldData, BTData);
          return 'C';
          break;
        case 'Q'://QuickRun
          strcpy(BTOldData, BTData);
          return 'Q';
          break;
        case 'R'://Run from settings
          strcpy(BTOldData, BTData);
          return 'R';
          break;
        case 'M'://Move to manual position



          strcpy(BTOldData, BTData);
          Serial.println(BTData);
          return 'M';
          break;
        case 'B':

          strcpy(BTOldData, BTData); //Clear Buffer
          return 'B';
          break;
        default:
          strcpy(BTOldData, BTData); //Otherwise
          break;
      }

    }///Here is where it happens only when data feed changes
    else {

      strcpy(BTOldData, BTData);
      newData = false;
      BTFeed[0] = '/0';
      return 'Z';
    }
  }
}
void Calibrate()
{
  //Move towards origin for initial calibration
  long initialhoming = -1;
  SliderSpeed = 3000;
  delay(1000);
  while (digitalRead(originendstop) == true)
  {
    MotorSlider.moveTo(initialhoming);
    MotorSlider.setSpeed(-ReturnOriginSpeed);
    initialhoming --;
    MotorSlider.runSpeed();
  }
  MotorSlider.setCurrentPosition(0);//Origin Reached
  initialhoming = 1;

  while (digitalRead(farendstop) == true)
  {
    MotorSlider.moveTo(initialhoming);
    MotorSlider.setSpeed(SliderSpeed);
    initialhoming ++;
    MotorSlider.runSpeed();
  }
  long positionthink = MotorSlider.currentPosition();//Estimation of steps calculated
  Serial.print("Position THinks Slider Pos is:");
  Serial.println(positionthink);
  MotorSlider.setCurrentPosition(0);
  delay(1000);
  while (digitalRead(originendstop) == true)
  {
    // Serial.println("Here");
    initialhoming --;
    MotorSlider.moveTo(initialhoming);
    MotorSlider.setSpeed(-SliderSpeed);

    MotorSlider.runSpeed();

  }
  long positionrethink = MotorSlider.currentPosition();
  MotorSlider.setCurrentPosition(0);
  Serial.print("Position THinks Slider Pos is:"); Serial.println(-positionrethink);
  CalibrationSliderPos = (positionthink + (-1) * positionrethink) / 2;
  Serial.print("Averaged Steps = :"); Serial.println(CalibrationSliderPos);
  delay(CalibrationRestTime);


  //CALIBRATED SLIDER
  //CALIBRATE PAN
  MotorPan.setCurrentPosition(0);
  PanSpeed = 800;
  MotorPan.setSpeed(PanSpeed);
  MotorPan.moveTo(CalibrationPanPos);
  while (MotorPan.distanceToGo() != 0)
  {
    MotorPan.run();
  }
  delay(CalibrationRestTime);
  Serial.print("Told to move:"); Serial.println(CalibrationPanPos);
  Serial.print("ACtually got:"); Serial.println(MotorPan.currentPosition());
  MotorPan.setSpeed(PanSpeed);
  MotorPan.moveTo(0);
  while (MotorPan.distanceToGo() != 0)
  {
    MotorPan.run();
  }
  delay(CalibrationRestTime);
  //Serial.print("Told to move:"); Serial.println(-CalibrationPanPos);
  //Serial.print("ACtually got:"); Serial.println(MotorPan.currentPosition());
  MotorPan.setCurrentPosition(0);
  PanSpeed = 400;

  //Calibrate Speed
  LengthToSpeedCalc();
}


void LengthToSpeedCalc()
{
  LengthToSpeedMultiplier = CalibrationSliderPos / 4;
}



void QuickMove()
{
  float speedscale = 1;
  long initialhoming = -1;
  SliderSpeed = 4000;
  delay(500);
  while (digitalRead(originendstop) == true)
  {
    MotorSlider.moveTo(initialhoming);
    MotorSlider.setSpeed(-ReturnOriginSpeed);
    initialhoming --;
    MotorSlider.runSpeed();
  }
  MotorSlider.setCurrentPosition(0);//Origin Reached
  MotorPan.setCurrentPosition(0);
  int stepsratio = round(CalibrationSliderPos / FeasiblePan) + 1;
  Serial.print("Steps:");
  Serial.println(stepsratio);
  int i = 1;

  SliderSpeed = LengthToSpeedMultiplier * speedscale;
  int stepcounter = 1;
  PanSpeed = SliderSpeed / stepsratio;
  //Serial.println(ToolPathArray);
  //MotorSlider.moveTo(CalibrationSliderPos);
  MotorSlider.setSpeed(SliderSpeed);
  //MotorPan.move(FeasiblePan);
  MotorPan.setSpeed(PanSpeed);
  //MotorPan.setAcceleration(100);
  //MotorSlider.setAcceleration(200);
  //Serial.print("Pan Initial Position:");Serial.println(MotorPan.currentPosition());
  //Serial.print("Pan Target Position:");Serial.println(MotorPan.distanceToGo());
  MotorSlider.moveTo(CalibrationSliderPos);
  MotorPan.moveTo(FeasiblePan);
  //Serial.print("Slide Initial Position:");Serial.println(MotorSlider.currentPosition());
  //Serial.print("Slide Target Position:");Serial.println(MotorSlider.distanceToGo());
  while (MotorPan.distanceToGo() >= 0) {
    MotorSlider.setSpeed(SliderSpeed); MotorPan.setSpeed(PanSpeed);
    MotorSlider.runSpeed();
    MotorPan.runSpeed();
  }

  /* for(i=1;i<(CalibrationSliderPos);i++)
    {
     MotorSlider.moveTo(i);
         if(i % stepsratio == 0){
           MotorPan.moveTo(stepcounter);
           stepcounter++;
           MotorPan.run();
         }


     MotorSlider.run();
     delay(1);
     //Serial.print("Pan:");Serial.println(MotorPan.currentPosition());
     //Serial.print("Slide:");Serial.println(MotorSlider.currentPosition());
    }*/

}







void ManualMoveToPosX(int Position, int Angle)
{
  //Return to origin
  long initialhoming = -1;
  SliderSpeed = 3000;
  delay(500);
  while (digitalRead(originendstop) == true)
  {
    MotorSlider.moveTo(initialhoming);
    MotorSlider.setSpeed(-ReturnOriginSpeed);
    initialhoming --;
    MotorSlider.runSpeed();
  }
  MotorSlider.setCurrentPosition(0);//Origin Reached
  //Serial.print("Pan Position:");Serial.println(MotorPan.currentPosition());


  int MovePos = round(Position * 0.01 * CalibrationSliderPos);
  Serial.print("Angle Recieved is:"); Serial.println(Angle);
  //Serial.println(CalibrationPanPos);
  int AnglePos = round(Angle * (CalibrationPanPos / 180)); //Scales up to correct so max rotation is 120 degrees
  Serial.print("Angle Position calculated:"); Serial.println(AnglePos);
  delay(500);




  MotorSlider.moveTo(MovePos);
  while (MotorSlider.distanceToGo() != 0)
  {
    MotorSlider.run();
  }



  //Pan Camera
  delay(500);

  //Serial.print("Current Start Pos:");Serial.println(MotorPan.currentPosition());
  MotorPan.moveTo(-AnglePos);
  
  while (MotorPan.distanceToGo() != 0)
  {
    MotorPan.setSpeed(-PanSpeed);
    MotorPan.runSpeed();
    //Serial.print("Current Pos:");Serial.println(MotorPan.currentPosition());
  }


  delay(120000);//WAIT FOR PERIOD OF TIME BEFORE RETURNING

  MotorPan.moveTo(0);
  MotorPan.setSpeed(PanSpeed);
  while (MotorPan.distanceToGo() != 0)
  {
    MotorPan.runSpeed();
    //Serial.print("Current Pos:");Serial.println(MotorPan.currentPosition());
  }

  MotorSlider.moveTo(0);
  MotorSlider.setSpeed(-SliderSpeed);
  while (MotorSlider.distanceToGo() != 0)
  {
    MotorSlider.runSpeed();
  }



}













void RunFromSettings()
{

  Serial.print("Recieved is:"); Serial.println(BTFeed);
  switch (BTFeed[1])
  {
    case 'P'://Pan Enabled
      TempSpeed[0] = BTData[2];
      TempSpeed[1] = BTData[3];
      MoveSpeed = atoi(TempSpeed);
      Serial.print("Move Speed:"); Serial.println(MoveSpeed);
      break;
  }
  switch(BTFeed[7])
  {
      case 's'://Slider Disabled
      TimeLapseMode = false;
      Serial.println("Time Lapse Disabled");
      break;
      case 'S'://Slider Enabled
      TimeLapseMode = true;
      Serial.println("Time Lapse Enabled");
      break;
  }


  long initialhoming = -1;
  float SpeedMove = MoveSpeed / SpeedScalar;
  int stepsratio = round(CalibrationSliderPos / FeasiblePan) + 2;
  SliderSpeed = 4000;
  delay(500);
  while (digitalRead(originendstop) == true)
  {
    MotorSlider.moveTo(initialhoming);
    MotorSlider.setSpeed(-ReturnOriginSpeed);
    initialhoming --;
    MotorSlider.runSpeed();
  }
  MotorSlider.setCurrentPosition(0);//Origin Reached
  MotorPan.setCurrentPosition(0);


  int i = 1;
  if(TimeLapseMode == true)
  {
    SliderSpeed = LengthToSpeedMultiplier * 0.02;
    stepsratio -= 2;
  }else
  {
    SliderSpeed = LengthToSpeedMultiplier * SpeedMove +100;
  }
  ;
  int stepcounter = 1;
  PanSpeed = SliderSpeed / stepsratio;
  //Serial.println(ToolPathArray);
  //MotorSlider.moveTo(CalibrationSliderPos);
  MotorSlider.setSpeed(SliderSpeed);
  //MotorPan.move(FeasiblePan);
  MotorPan.setSpeed(PanSpeed);
  //MotorPan.setAcceleration(100);
  //MotorSlider.setAcceleration(200);
  //Serial.print("Pan Initial Position:");Serial.println(MotorPan.currentPosition());
  //Serial.print("Pan Target Position:");Serial.println(MotorPan.distanceToGo());
  MotorSlider.moveTo(CalibrationSliderPos);
  MotorPan.moveTo(FeasiblePan);
  //Serial.print("Slide Initial Position:");Serial.println(MotorSlider.currentPosition());
  //Serial.print("Slide Target Position:");Serial.println(MotorSlider.distanceToGo());
  while (MotorPan.distanceToGo() >= 0) {
    MotorSlider.setSpeed(SliderSpeed); MotorPan.setSpeed(PanSpeed);
    MotorSlider.runSpeed();
    MotorPan.runSpeed();
  }


}





void loop() {
  // put your main code here, to run repeatedly:
  //Serial.println("At STart of loop");
  recvWithEndMarker();

  switch (CheckNewData())
  {
    case 'C':
      Serial.println("Calibrating");
      Calibrate();
      Serial.println("Calbrated");
      //delay(1000);
      break;
    case 'Q':

      Serial.println("Quick Settings");
      QuickMove();
      //delay(1000);
      break;
    case 'R':

      Serial.println("Run from Settings");
      RunFromSettings();
      //delay(1000);
      break;
    case 'M':
      //Serial.print("All is:");Serial.println(BTData);
      TempManMove[0] = BTData[2];
      TempManMove[1] = BTData[3];
      int tempval1 =  BTData[7] - 48;
      int tempval2 =  BTData[8] - 48;
      int TempVal = abs(tempval1) * 10 + abs(tempval2);
      Serial.print("Temp Vals :");
      Serial.println(TempVal);
      ManualMoveToPosX(atoi(TempManMove), TempVal);

      //delay(1000);
      break;
    case 'B':
      Serial.println("YOU PRESSED B");
      //delay(1000);
      break;
    case 'Z':
      break;
    default:
      //delay(1000);
      break;
  }
}
